@@ -606,6 +606,9 @@ static const efi_config_table_type_t common_tables[] __initconst = {
 #endif
 #ifdef CONFIG_UNACCEPTED_MEMORY
 	{LINUX_EFI_UNACCEPTED_MEM_TABLE_GUID,	&efi.unaccepted,	"Unaccepted"	},
+#endif
+#ifdef CONFIG_EFI_GENERIC_STUB
+	{LINUX_EFI_SCREEN_INFO_TABLE_GUID,	&screen_info_table			},
 #endif
 	{},
 };
@@ -1067,6 +1070,7 @@ int efi_status_to_err(efi_status_t status)
 		return -EINVAL;
 	return found->errno;
 }
+EXPORT_SYMBOL_GPL(efi_status_to_err);
 
 const char *
 efi_status_to_str(efi_status_t status)
