@@ -73,8 +73,8 @@ static inline void set_arch_panic_timeout(int timeout, int arch_default_timeout)
 #define TAINT_AUX			16
 #define TAINT_RANDSTRUCT		17
 #define TAINT_TEST			18
+#define TAINT_FWCTL			19
 /* Start of Red Hat-specific taint flags */
-#define TAINT_19			19
 #define TAINT_20			20
 #define TAINT_21			21
 #define TAINT_22			22
