@@ -242,8 +242,8 @@ struct power_supply_config {
 struct power_supply_desc {
 	const char *name;
 	enum power_supply_type type;
-	const enum power_supply_usb_type *usb_types;
-	size_t num_usb_types;
+	u8 charge_behaviours;
+	u32 usb_types;
 	const enum power_supply_property *properties;
 	size_t num_properties;
 
