@@ -215,7 +215,7 @@ static void panic_print_sys_info(bool console_flush)
 		show_state();
 
 	if (panic_print & PANIC_PRINT_MEM_INFO)
-		show_mem(0, NULL);
+		show_mem();
 
 	if (panic_print & PANIC_PRINT_TIMER_INFO)
 		sysrq_timer_list_show();
@@ -489,7 +489,7 @@ const struct taint_flag taint_flags[TAINT_FLAGS_COUNT] = {
 	[ TAINT_AUX ]			= { 'X', ' ', true },
 	[ TAINT_RANDSTRUCT ]		= { 'T', ' ', true },
 	[ TAINT_TEST ]			= { 'N', ' ', true },
-	[ TAINT_19 ]			= { '?', '-', false },
+	[ TAINT_FWCTL ]			= { 'J', ' ', true },
 	[ TAINT_20 ]			= { '?', '-', false },
 	[ TAINT_21 ]			= { '?', '-', false },
 	[ TAINT_22 ]			= { '?', '-', false },
