@@ -1240,7 +1240,7 @@ static int sta350_i2c_remove(struct i2c_client *client)
 }
 
 static const struct i2c_device_id sta350_i2c_id[] = {
-	{ "sta350", 0 },
+	{ "sta350" },
 	{ }
 };
 MODULE_DEVICE_TABLE(i2c, sta350_i2c_id);
